@@ -1,7 +1,7 @@
 /* RFC 2425 MIME Directory Attribute Object
  * Copyright (C) 2002-2005  Sebastian Rittau <srittau@jroger.in-berlin.de>
  *
- * $Id: mimedir-attribute.c,v 1.36.2.2 2005/11/03 17:04:21 srittau Exp $
+ * $Id: mimedir-attribute.c 257 2006-08-15 21:16:18Z srittau $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -269,8 +269,11 @@ mimedir_attribute_get_property (GObject		*object,
  */
 
 #if 0
+/* Convert an integer value in the range 0 <= i <= 63 to the appropriate
+ * Base64 character.
+ */
 static gchar
-int_to_char (guint i, GError **error)
+base64_int_to_char (guint i)
 {
 	g_assert (i <= 63);
 
@@ -287,13 +290,19 @@ int_to_char (guint i, GError **error)
 }
 #endif
 
+/* Convert a Base64 character to the appropriate int value 0 <= i <= 63. If
+ * the supplied character is the Base64 fill character '=', return 64. If
+ * the supplied character is not a valid Base64 character, return 65.
+ */
 static guint
-char_to_int (gchar c, GError **error)
+base64_char_to_int (gchar c)
 {
 	if (c >= 'A' && c <= 'Z')
 		return c - 'A';
 	else if (c >= 'a' && c <= 'z')
 		return c - 'a' + 26;
+	else if (c >= '0' && c <= '9')
+		return c - '0' + 52;
 	else if (c == '+')
 		return 62;
 	else if (c == '/')
@@ -301,8 +310,52 @@ char_to_int (gchar c, GError **error)
 	else if (c == '=')
 		return 64;
 
-	g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_BASE64, MIMEDIR_ATTRIBUTE_ERROR_INVALID_BASE64_STR);
-	return 0;
+	return 65;
+}
+
+/* This is an ad-hoc check for Base64 encoding. It takes a
+ * string as input, which is supposed to be the first line of an
+ * attribute. If this line contains either the property ENCODING=b
+ * or the property ENCODING=BASE64, this function returns TRUE,
+ * otherwise it returns FALSE.
+ *
+ * This hack is needed, because Base64 encoded attributes break the normal
+ * line folding mechanism.
+ */
+static gboolean
+base64_check (const gchar *line)
+{
+	GString *token;
+	guint mode;
+	gboolean res;
+
+	token = g_string_new ("");
+
+	mode = 0; /* 0: parsing attribute name; 1: parsing property */
+	for (; *line != '\0'; line++) {
+		if (*line == ':')
+			break;
+		else if (*line == ';') {
+			if (mode == 1) {
+				if (!g_ascii_strcasecmp (token->str, "ENCODING=b") ||
+				    !g_ascii_strcasecmp (token->str, "ENCODING=BASE64")) {
+					g_string_free (token, TRUE);
+					return TRUE;
+				}
+				g_string_free (token, TRUE);
+				token = g_string_new ("");
+			} else
+				mode = 1;
+		} else if (mode == 1)
+			g_string_append_c (token, *line);
+	}
+
+	res = g_ascii_strcasecmp (token->str, "ENCODING=b") == 0 ||
+	      g_ascii_strcasecmp (token->str, "ENCODING=BASE64") == 0;
+
+	g_string_free (token, TRUE);
+
+	return res;
 }
 
 #if 0
@@ -328,32 +381,20 @@ base64_encode (const gchar *string, GError **error)
 		c3 = ((s2 % 0x10) << 2) + (s3 >> 6);
 		c4 = s3 % 0x40;
 
-		g_string_append_c (encoded, int_to_char (c1, error));
-		if (error) {
-			g_string_free (encoded, TRUE);
-			return NULL;
-		}
-		g_string_append_c (encoded, int_to_char (c2, error));
-		if (error) {
-			g_string_free (encoded, TRUE);
-			return NULL;
-		}
-		g_string_append_c (encoded, size >= 2 ? int_to_char (c3, error) : '=');
-		if (error) {
-			g_string_free (encoded, TRUE);
-			return NULL;
-		}
-		g_string_append_c (encoded, size >= 3 ? int_to_char (c4, error) : '=');
-		if (error) {
-			g_string_free (encoded, TRUE);
-			return NULL;
-		}
+		g_string_append_c (encoded, base64_int_to_char (c1));
+		g_string_append_c (encoded, base64_int_to_char (c2));
+		g_string_append_c (encoded, size >= 2 ? base64_int_to_char (c3) : '=');
+		g_string_append_c (encoded, size >= 3 ? base64_int_to_char (c4) : '=');
 	}
 
 	return g_string_free (encoded, FALSE);
 }
 #endif
 
+/* Decode the supplied Base64 string. Return a newly allocated buffer that
+ * must be freed by the called using g_free(). If there are errors while
+ * decoding the string, return NULL and set error accordingly.
+ */
 static gchar *
 base64_decode (const gchar *string, GError **error)
 {
@@ -364,44 +405,30 @@ base64_decode (const gchar *string, GError **error)
 	decoded = g_string_sized_new (size * 3 / 4 + 3);
 
 	for (; size >= 4; size -= 4, buffer += 4) {
-		guint  i1, i2, i3, i4;
+		int i;
+		guint is[4]; /* decoded characters */
 		guint8 c1, c2, c3;
 
-		i1 = char_to_int (buffer[0], error);
-		if (error) {
-			g_string_free (decoded, TRUE);
-			return NULL;
-		}
-		i2 = char_to_int (buffer[1], error);
-		if (error) {
-			g_string_free (decoded, TRUE);
-			return NULL;
-		}
-		i3 = char_to_int (buffer[2], error);
-		if (error) {
-			g_string_free (decoded, TRUE);
-			return NULL;
-		}
-		i4 = char_to_int (buffer[3], error);
-		if (error) {
-			g_string_free (decoded, TRUE);
-			return NULL;
-		}
+		for (i = 0; i < 4; i++)
+			is[i] = base64_char_to_int (buffer[i]);
 
-		if (i1 == 64 || i2 == 64) {
+		/* The first two characters of each four character block
+		 * must not be '='.
+		 */
+		if (is[0] > 63 || is[1] > 63 || is[2] > 64 || is[3] > 64) {
 			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_BASE64, MIMEDIR_ATTRIBUTE_ERROR_INVALID_BASE64_STR);
 			g_string_free (decoded, TRUE);
 			return NULL;
 		}
 
-		c1 = (i1 << 2) + (i2 >> 4);
-		c2 = ((i2 % 0x10) << 4) + (i3 == 0x40 ? 0 : (i3 >> 2));
-		c3 = (i3 == 0x40 ? 0 : ((i3 % 4) << 6)) + (i4 == 0x40 ? 0 : i4);
+		c1 = (is[0] << 2) + (is[1] >> 4);
+		c2 = ((is[1] % 0x10) << 4) + (is[2] == 0x40 ? 0 : (is[2] >> 2));
+		c3 = (is[2] == 0x40 ? 0 : ((is[2] % 4) << 6)) + (is[3] == 0x40 ? 0 : is[3]);
 
 		g_string_append_c (decoded, c1);
-		if (i3 != 64)
+		if (is[2] != 64)
 			g_string_append_c (decoded, c2);
-		if (i3 != 64 && i4 != 64)
+		if (is[2] != 64 && is[3] != 64)
 			g_string_append_c (decoded, c3);
 	}
 
@@ -538,6 +565,9 @@ escape (const gchar *text, GError **error)
 
 	for (; text[0] != '\0'; text++) {
 		switch (text[0]) {
+		case ';':
+			g_string_append (string, "\\;");
+			break;
 		case ',':
 			g_string_append (string, "\\,");
 			break;
@@ -672,7 +702,8 @@ mimedir_attribute_internal_append_parameter (MIMEDirAttribute *attribute, const
 			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_PARAMETER_NOT_UNIQUE, MIMEDIR_ATTRIBUTE_ERROR_PARAMETER_NOT_UNIQUE_STR, name, attribute->priv->name);
 			return FALSE;
 		}
-		if (!g_ascii_strcasecmp ((const gchar *) values->data, "b"))
+		if (!g_ascii_strcasecmp ((const gchar *) values->data, "b") ||
+		    !g_ascii_strcasecmp ((const gchar *) values->data, "base64"))
 			attribute->priv->encoding = MIMEDIR_ATTRIBUTE_ENCODING_BASE64;
 		else if (!g_ascii_strcasecmp ((const gchar *) values->data, "quoted-printable"))
 			attribute->priv->encoding = MIMEDIR_ATTRIBUTE_ENCODING_QP;
@@ -834,6 +865,7 @@ get_attribute_string (const gchar *string, GString *gstring)
 {
 	ssize_t pos;
 	gboolean qp;
+	gboolean basesixfour;
 
 	g_string_truncate (gstring, 0);
 
@@ -842,6 +874,7 @@ get_attribute_string (const gchar *string, GString *gstring)
 	pos = get_line (string, gstring);
 
 	qp = qp_check (gstring->str);
+	basesixfour = base64_check (gstring->str);
 
 	/* Retrieve folded lines */
 
@@ -856,6 +889,13 @@ get_attribute_string (const gchar *string, GString *gstring)
 
 			if (string[pos] == ' ' || string[pos] == '\t')
 				pos++;
+		} else if (basesixfour) {
+			if (string[pos] == ' ' || string[pos] == '\t')
+				pos++;
+			else if (string[pos] == '\r' || string[pos] == '\n')
+				break;
+			else if (string[pos] == '\0')
+				break;
 		} else {
 
 			/* If next byte is not a white space, we're done */
@@ -1380,7 +1420,7 @@ mimedir_attribute_get_parameter_value (MIMEDirAttribute *attribute, const gchar
 	g_return_val_if_fail (parameter != NULL, FALSE);
 
 	p = mimedir_attribute_get_parameter (attribute, parameter);
-	if (p)
+	if (p && p->values)
 		return (gchar *) p->values->data;
 	else
 		return NULL;
@@ -1731,7 +1771,7 @@ mimedir_attribute_get_value_text_list (MIMEDirAttribute *attribute, GError **err
 			}
 			break;
 		case ',':
-			g_slist_append (list, g_string_free (string, FALSE));
+			list = g_slist_append (list, g_string_free (string, FALSE));
 			string = g_string_new ("");
 			break;
 		case '\r':
