@@ -1,7 +1,7 @@
 /* RFC 2425 MIME Directory Profile Object
  * Copyright (C) 2002-2005  Sebastian Rittau <srittau@jroger.in-berlin.de>
  *
- * $Id: mimedir-profile.c,v 1.18 2005/02/26 22:46:04 srittau Exp $
+ * $Id: mimedir-profile.c 176 2005-02-26 22:46:04Z srittau $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -589,11 +589,13 @@ mimedir_profile_write_to_string (MIMEDirProfile *profile)
 	if (!mimedir_profile_write_to_channel (profile, channel, &error)) {
 		g_free (tmpname);
 		g_io_channel_unref (channel);
+		close(fd);
 		g_warning (error->message);
 		g_error_free (error);
 		return g_strdup ("");
 	}
 	g_io_channel_unref (channel);
+	close(fd);
 
 	if (!g_file_get_contents (tmpname, &buffer, NULL, &error)) {
 		g_free (tmpname);
@@ -602,6 +604,7 @@ mimedir_profile_write_to_string (MIMEDirProfile *profile)
 		return g_strdup ("");
 	}
 
+	g_unlink(tmpname);
 	g_free (tmpname);
 
 	return buffer;
