@@ -1,7 +1,7 @@
 /* RFC 2445 iCalendar MIME Directory Profile Object
  * Copyright (C) 2002-2005  Sebastian Rittau <srittau@jroger.in-berlin.de>
  *
- * $Id: mimedir-vcal.c,v 1.14 2005/03/01 03:01:49 srittau Exp $
+ * $Id: mimedir-vcal.c 185 2005-03-01 03:01:49Z srittau $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -769,18 +769,6 @@ mimedir_vcal_read_from_profile (MIMEDirVCal *vcal, MIMEDirProfile *profile, GErr
 			priv->components = g_list_append (priv->components, tz);
 		}
 
-		else if (!g_ascii_strcasecmp (name, "VALARM")) {
-			MIMEDirVAlarm *alarm;
-
-			alarm = mimedir_valarm_new_from_profile (profile, error);
-			if (!alarm) {
-				g_free (name);
-				return FALSE;
-			}
-
-			priv->components = g_list_append (priv->components, alarm);
-		}
-
 		g_free (name);
 	}
 
@@ -1010,7 +998,10 @@ mimedir_vcal_get_event_list (MIMEDirVCal *vcal)
 
 	for (item = vcal->priv->components; item != NULL; item = g_list_next (item)) {
 		if (MIMEDIR_IS_VEVENT (item->data))
+		  {
+			g_object_ref(item->data);
 			list = g_slist_append (list, item->data);
+		  }
 	}
 
 	return list;
@@ -1036,7 +1027,10 @@ mimedir_vcal_get_todo_list (MIMEDirVCal *vcal)
 
 	for (item = vcal->priv->components; item != NULL; item = g_list_next (item)) {
 		if (MIMEDIR_IS_VTODO (item->data))
+		  {
+			g_object_ref (item->data);
 			list = g_slist_append (list, item->data);
+		  }
 	}
 
 	return list;
