@@ -1,7 +1,8 @@
 /* RFC 2445 iCal Component Object
  * Copyright (C) 2002-2005  Sebastian Rittau <srittau@jroger.in-berlin.de>
+ * Copyright (C) 2007  Graham Cobb <g+770@cobb.uk.net>
  *
- * $Id: mimedir-vcomponent.c,v 1.26.2.1 2005/11/03 17:04:21 srittau Exp $
+ * $Id: mimedir-vcomponent.c 247 2005-11-26 15:40:20Z srittau $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -419,7 +420,7 @@ mimedir_vcomponent_class_init (MIMEDirVComponentClass *klass)
 				  _("Trigger"),
 				  _("Trigger"),
 				  G_MININT, G_MAXINT, 0,
-				  G_PARAM_READABLE);
+				  G_PARAM_READWRITE);
 	g_object_class_install_property (gobject_class, PROP_TRIGGER, pspec);
 	pspec = g_param_spec_object ("trigger-datetime",
 				     _("Trigger date/time"),
@@ -456,7 +457,7 @@ mimedir_vcomponent_class_init (MIMEDirVComponentClass *klass)
 				   _("Sequence"),
 				   _("Sequence number"),
 				   0, G_MAXINT, 0,
-				   G_PARAM_READABLE);
+				   G_PARAM_READWRITE);
 	g_object_class_install_property (gobject_class, PROP_SEQ, pspec);
 }
 
@@ -684,6 +685,9 @@ mimedir_vcomponent_set_property (GObject	*object,
 		g_object_ref (G_OBJECT (priv->last_modified));
 		break;
 	}
+	case PROP_SEQ:
+		priv->sequence = g_value_get_uint (value);
+	break;
 
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -905,6 +909,7 @@ mimedir_vcomponent_get_value_duration (MIMEDirAttribute *attr, GError **error)
 	const gchar *s;
 	gint duration = 0;
 	gboolean neg = FALSE;
+	gboolean dur_week = FALSE;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, 0);
 
@@ -936,57 +941,61 @@ mimedir_vcomponent_get_value_duration (MIMEDirAttribute *attr, GError **error)
 
 		if (s[0] == 'D' || s[0] == 'd')
 			duration = num * SECS_PER_DAY;
-		else if (s[0] == 'W' || s[0] == 'w')
+		else if (s[0] == 'W' || s[0] == 'w') {
 			duration = num * SECS_PER_WEEK;
-		else {
+			dur_week = TRUE;
+		} else {
 			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT_STR, "DURATION", name);
 			return 0;
 		}
 		s++;
 	}
 
-	if (s[0] == 'T' || s[0] == 't') {
+	if ((s[0] == 'T' || s[0] == 't') && !dur_week) {
+	  /* Parse a dur-time */
 		GError *err = NULL;
 		guint num;
 
 		s++;
 
+		/* Must begin with a number */
 		num = mimedir_vcomponent_parse_number (name, "DURATION", &s, &err);
 		if (err) {
 			g_propagate_error (error, err);
 			return 0;
 		}
-		if (s[0] != 'H') {
-			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT_STR, "DURATION", name);
-			return 0;
-		}
-		s++;
-		duration += num * SECS_PER_HOUR;
+		/* Is it dur-hour? */
+		if (s[0] == 'H') {
+		  s++;
+		  duration += num * SECS_PER_HOUR;
 
-		if (s[0]) {
+		  /* Got dur-hour, any more? */
+		  if (s[0]) {
 			num = mimedir_vcomponent_parse_number (name, "DURATION", &s, &err);
 			if (err) {
 				g_propagate_error (error, err);
 				return 0;
 			}
-			if (s[0] != 'M') {
-				g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT_STR, "DURATION", name);
-				return 0;
-			}
+		  }
+		}
+
+		/* Is it dur-minute? */
+		if (s[0] == 'M') {
 			s++;
 			duration += num * SECS_PER_MINUTE;
-		}
 
-		if (s[0]) {
-			num = mimedir_vcomponent_parse_number (name, "DURATION", &s, &err);
-			if (err) {
+			/* Got dur-minute, any more? */
+			if (s[0]) {
+			  num = mimedir_vcomponent_parse_number (name, "DURATION", &s, &err);
+			  if (err) {
 				g_propagate_error (error, err);
 				return 0;
+			  }
 			}
-			if (s[0] != 'S') {
-				g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT, MIMEDIR_ATTRIBUTE_ERROR_INVALID_FORMAT_STR, "DURATION", name);
-				return 0;
-			}
+		}
+
+		/* Is it dur-second? */
+		if (s[0] == 'S') {
 			s++;
 			duration += num;
 		}
@@ -1870,11 +1879,10 @@ mimedir_vcomponent_parse_attribute (MIMEDirVComponent *vcomponent, MIMEDirAttrib
 
 		par = mimedir_attribute_get_parameter_value (attr, "VALUE");
 		if (par && !g_ascii_strcasecmp (par, "DATE-TIME")) {
+			/* Note: RFC says that is DATE-TIME is specified, value must be in UTC */
 			priv->trigger_dt = mimedir_attribute_get_value_datetime (attr, error);
 			if (!priv->trigger_dt)
 				return FALSE;
-
-			/* FIXME: honor tz parameter */
 		} else {
 			priv->trigger = mimedir_vcomponent_get_value_duration (attr, &err);
 			if (err) {
@@ -1883,7 +1891,7 @@ mimedir_vcomponent_parse_attribute (MIMEDirVComponent *vcomponent, MIMEDirAttrib
 			}
 
 			par = mimedir_attribute_get_parameter_value (attr, "RELATED");
-			if (!g_ascii_strcasecmp (par, "END"))
+			if (par && !g_ascii_strcasecmp (par, "END"))
 				priv->trigger_end = TRUE;
 		}
 	}
@@ -1956,10 +1964,12 @@ mimedir_vcomponent_parse_attribute (MIMEDirVComponent *vcomponent, MIMEDirAttrib
 			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_VALUE, MIMEDIR_ATTRIBUTE_ERROR_INVALID_VALUE_STR, name);
 			return FALSE;
 		}
-		if (priv->last_modified->timezone != MIMEDIR_DATETIME_UTC) {
-			g_set_error (error, MIMEDIR_ATTRIBUTE_ERROR, MIMEDIR_ATTRIBUTE_ERROR_INVALID_VALUE, MIMEDIR_ATTRIBUTE_ERROR_INVALID_VALUE_STR, name);
-			return FALSE;
-		}
+		/* RFC 2445 requires the LAST-MODIFIED field to be in UTC
+		 * time format (section 4.8.7.3). Unfortunately some software
+		 * does not conform to this requirement.
+		 */
+		if (priv->last_modified->timezone != MIMEDIR_DATETIME_UTC)
+			mimedir_datetime_to_utc (priv->last_modified);
 	}
 
 	/* Sequence Number (4.8.7.4) */
@@ -2224,7 +2234,7 @@ gboolean
 mimedir_vcomponent_read_from_profile (MIMEDirVComponent *vcomponent, MIMEDirProfile *profile, GError **error)
 {
 	MIMEDirVComponentPriv *priv;
-	GSList *attrs;
+	GSList *attrs, *components;
 
 	g_return_val_if_fail (vcomponent != NULL, FALSE);
 	g_return_val_if_fail (MIMEDIR_IS_VCOMPONENT (vcomponent), FALSE);
@@ -2246,7 +2256,31 @@ mimedir_vcomponent_read_from_profile (MIMEDirVComponent *vcomponent, MIMEDirProf
 			return FALSE;
 	}
 
-	/* FIXME: read sub-profiles (vAlarms) */
+	/* Read sub-profiles (vAlarms) */
+	components = mimedir_profile_get_subprofiles (profile);
+
+	for (; components != NULL; components = g_slist_next (components)) {
+		MIMEDirProfile *profile;
+		gchar *name;
+
+		profile = MIMEDIR_PROFILE (components->data);
+
+		g_object_get (G_OBJECT (profile), "name", &name, NULL);
+
+		if (!g_ascii_strcasecmp (name, "VALARM")) {
+			MIMEDirVAlarm *alarm;
+
+			alarm = mimedir_valarm_new_from_profile (profile, error);
+			if (!alarm) {
+				g_free (name);
+				return FALSE;
+			}
+
+			priv->alarms = g_list_prepend(priv->alarms, alarm);
+		}
+
+		g_free (name);
+	}
 
 	/* Validity checks */
 
@@ -2994,7 +3028,7 @@ mimedir_vcomponent_set_location (MIMEDirVComponent *vcomponent, const gchar *loc
 	g_free (priv->location);
 	g_free (priv->location_alt);
 	priv->location     = g_strdup (location);
-	priv->location_alt = g_strdup (uri);
+	priv->location_alt = uri ? g_strdup (uri) : NULL;
 }
 
 /**
